@@ -46,6 +46,7 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC (SysprofCaptureReader, sysprof_capture_reader_unre
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (SysprofCaptureWriter, sysprof_capture_writer_unref)
 
 static gboolean no_decode;
+static gboolean enable_debuginfod;
 static GMainLoop *main_loop;
 static SysprofRecording *active_recording;
 
@@ -303,6 +304,7 @@ main (int   argc,
   gboolean scheduler_details = FALSE;
   gboolean system_bus = FALSE;
   gboolean session_bus = FALSE;
+  int stack_size = 0;
   int pid = -1;
   int fd;
   int flags;
@@ -335,6 +337,8 @@ main (int   argc,
     { "version", 0, 0, G_OPTION_ARG_NONE, &version, N_("Print the sysprof-cli version and exit") },
     { "buffer-size", 0, 0, G_OPTION_ARG_INT, &n_buffer_pages, N_("The size of the buffer in pages (1 = 1 page)") },
     { "monitor-bus", 0, 0, G_OPTION_ARG_STRING_ARRAY, &monitor_bus, N_("Additional D-Bus address to monitor") },
+    { "stack-size", 0, 0, G_OPTION_ARG_INT, &stack_size, N_("Stack size to copy for unwinding in user-space") },
+    { "no-debuginfod", 0, 0, G_OPTION_ARG_NONE, &enable_debuginfod, N_("Do not use debuginfod to resolve symbols") },
     { NULL }
   };
 
@@ -379,6 +383,10 @@ Examples:\n\
 \n\
   # Merge multiple syscap files into one\n\
   sysprof-cli --merge a.syscap b.syscap > c.syscap\n\
+\n\
+  # Unwind by capturing stack/register contents instead of frame-pointers\n\
+  # where the stack-size is a multiple of page-size\n\
+  sysprof-cli --stack-size=8192\n\
 "));
 
   if (!g_option_context_parse (context, &argc, &argv, &error))
@@ -533,13 +541,22 @@ Examples:\n\
   sysprof_profiler_add_instrument (profiler, sysprof_system_logs_new ());
 
   if (!no_perf)
-    sysprof_profiler_add_instrument (profiler, sysprof_sampler_new ());
+    {
+      if (stack_size == 0)
+        sysprof_profiler_add_instrument (profiler, sysprof_sampler_new ());
+      else
+        sysprof_profiler_add_instrument (profiler, sysprof_user_sampler_new (stack_size));
+    }
 
   if (!no_disk)
     sysprof_profiler_add_instrument (profiler, sysprof_disk_usage_new ());
 
   if (!no_decode)
-    sysprof_profiler_add_instrument (profiler, sysprof_symbols_bundle_new ());
+    {
+      SysprofInstrument *bundle = sysprof_symbols_bundle_new ();
+      sysprof_symbols_bundle_set_enable_debuginfod (SYSPROF_SYMBOLS_BUNDLE (bundle), enable_debuginfod);
+      sysprof_profiler_add_instrument (profiler, g_steal_pointer (&bundle));
+    }
 
   if (!no_cpu)
     sysprof_profiler_add_instrument (profiler, sysprof_cpu_usage_new ());
