@@ -31,6 +31,8 @@
 #include "sysprof-power-profiles.h"
 #include "sysprof-recording-pad.h"
 #include "sysprof-recording-template.h"
+#include "sysprof-stack-size.h"
+#include "sysprof-util.h"
 #include "sysprof-window.h"
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (SysprofCaptureWriter, sysprof_capture_writer_unref)
@@ -40,6 +42,7 @@ struct _SysprofGreeter
   AdwWindow                 parent_instance;
 
   GtkStringList            *envvars;
+  SysprofRecordingTemplate *recording_template;
 
   AdwViewStack             *view_stack;
   GtkListBox               *sidebar_list_box;
@@ -54,12 +57,35 @@ struct _SysprofGreeter
   GtkSwitch                *record_session_bus;
   GtkSwitch                *record_system_bus;
   GtkSwitch                *bundle_symbols;
+  GtkSwitch                *debuginfod;
   GtkButton                *record_to_memory;
   AdwComboRow              *power_combo;
-  SysprofRecordingTemplate *recording_template;
+  AdwComboRow              *sample_user_stack_size;
+  AdwExpanderRow           *user_stacks;
+  GtkLabel                 *user_stacks_caption;
 };
 
-G_DEFINE_FINAL_TYPE (SysprofGreeter, sysprof_greeter, ADW_TYPE_WINDOW)
+static GObject *
+sysprof_greeter_get_internal_child (GtkBuildable *buildable,
+                                    GtkBuilder   *builder,
+                                    const char   *name)
+{
+  SysprofGreeter *self = SYSPROF_GREETER (buildable);
+
+  if (g_strcmp0 (name, "template") == 0)
+    return G_OBJECT (self->recording_template);
+
+  return NULL;
+}
+
+static void
+buildable_iface_init (GtkBuildableIface *iface)
+{
+  iface->get_internal_child = sysprof_greeter_get_internal_child;
+}
+
+G_DEFINE_FINAL_TYPE_WITH_CODE (SysprofGreeter, sysprof_greeter, ADW_TYPE_WINDOW,
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE, buildable_iface_init))
 
 #define STRV_INIT(...) (const char * const[]){__VA_ARGS__,NULL}
 
@@ -162,7 +188,7 @@ sysprof_greeter_record_cb (GObject      *object,
     }
   else
     {
-      GtkWidget *pad = sysprof_recording_pad_new (recording);
+      GtkWidget *pad = sysprof_recording_pad_new (recording, self->recording_template);
 
       gtk_window_present (GTK_WINDOW (pad));
     }
@@ -175,11 +201,31 @@ sysprof_greeter_create_profiler (SysprofGreeter  *self,
                                  GError         **error)
 {
   g_autoptr(SysprofProfiler) profiler = NULL;
+  g_autoptr(GFile) state_file = NULL;
+  g_autoptr(GFile) dir = NULL;
   GtkStringObject *strobj;
   const char *str;
 
   g_assert (SYSPROF_IS_GREETER (self));
 
+  if (g_list_model_get_n_items (G_LIST_MODEL (self->envvars)))
+    {
+      g_autoptr(GStrvBuilder) envvars_builder = NULL;
+      g_auto(GStrv) envvars_list = NULL;
+      
+      envvars_builder = g_strv_builder_new ();
+
+      for (guint i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (self->envvars)); i++)
+        {
+          g_autoptr(GtkStringObject) env_strobj = g_list_model_get_item (G_LIST_MODEL (self->envvars), i);
+          g_strv_builder_add (envvars_builder, gtk_string_object_get_string (env_strobj));
+        }
+      envvars_list = g_strv_builder_end (envvars_builder);
+      g_object_set (self->recording_template,
+                    "environ", envvars_list,
+                    NULL);
+    }
+
   if ((strobj = adw_combo_row_get_selected_item (self->power_combo)) &&
       (str = gtk_string_object_get_string (strobj)))
     g_object_set (self->recording_template,
@@ -189,6 +235,17 @@ sysprof_greeter_create_profiler (SysprofGreeter  *self,
   if (!(profiler = sysprof_recording_template_apply (self->recording_template, error)))
     return NULL;
 
+  state_file = _get_default_state_file ();
+  dir = g_file_get_parent (state_file);
+
+  if (!g_file_query_exists (dir, NULL))
+    {
+      const char *path = g_file_peek_path (dir);
+      g_mkdir_with_parents (path, 0750);
+    }
+
+  sysprof_recording_template_save (self->recording_template, state_file, NULL);
+
   return g_steal_pointer (&profiler);
 }
 
@@ -310,7 +367,9 @@ sysprof_greeter_select_file_action (GtkWidget  *widget,
                                     const char *action_name,
                                     GVariant   *param)
 {
-  sysprof_window_open_file (GTK_WINDOW (widget));
+  SysprofGreeter *self = SYSPROF_GREETER (widget);
+
+  sysprof_window_open_file (GTK_WINDOW (widget), self->recording_template);
 }
 
 static char *
@@ -455,6 +514,26 @@ translate_power_profile (GtkStringObject *strobj)
   return g_strdup (str);
 }
 
+static void
+on_stack_size_changed_cb (SysprofGreeter *self,
+                          GParamSpec     *pspec,
+                          AdwComboRow    *row)
+{
+  GObject *item;
+
+  g_assert (SYSPROF_IS_GREETER (self));
+  g_assert (ADW_IS_COMBO_ROW (row));
+
+  if ((item = adw_combo_row_get_selected_item (row)))
+    {
+      guint stack_size = sysprof_stack_size_get_size (SYSPROF_STACK_SIZE (item));
+
+      g_object_set (self->recording_template,
+                    "stack-size", stack_size,
+                    NULL);
+    }
+}
+
 static void
 sysprof_greeter_dispose (GObject *object)
 {
@@ -463,6 +542,7 @@ sysprof_greeter_dispose (GObject *object)
   gtk_widget_dispose_template (GTK_WIDGET (self), SYSPROF_TYPE_GREETER);
 
   g_clear_object (&self->envvars);
+  g_clear_object (&self->recording_template);
 
   G_OBJECT_CLASS (sysprof_greeter_parent_class)->dispose (object);
 }
@@ -479,6 +559,7 @@ sysprof_greeter_class_init (SysprofGreeterClass *klass)
 
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, app_environment);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, bundle_symbols);
+  gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, debuginfod);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, envvars);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, power_combo);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, record_compositor);
@@ -489,10 +570,12 @@ sysprof_greeter_class_init (SysprofGreeterClass *klass)
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, record_system_bus);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, record_system_logs);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, record_to_memory);
-  gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, recording_template);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, sample_javascript_stacks);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, sample_native_stacks);
+  gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, sample_user_stack_size);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, sidebar_list_box);
+  gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, user_stacks);
+  gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, user_stacks_caption);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, view_stack);
 
   gtk_widget_class_bind_template_callback (widget_class, sidebar_row_activated_cb);
@@ -507,14 +590,19 @@ sysprof_greeter_class_init (SysprofGreeterClass *klass)
 
   g_type_ensure (SYSPROF_TYPE_ENTRY_POPOVER);
   g_type_ensure (SYSPROF_TYPE_RECORDING_TEMPLATE);
+  g_type_ensure (SYSPROF_TYPE_STACK_SIZE);
 }
 
 static void
 sysprof_greeter_init (SysprofGreeter *self)
 {
   g_autoptr(GListModel) power_profiles = sysprof_power_profiles_new ();
+  g_autoptr(GFile) state_file = _get_default_state_file ();
   GtkListBoxRow *row;
 
+  if (!(self->recording_template = sysprof_recording_template_new_from_file (state_file, NULL)))
+    self->recording_template = sysprof_recording_template_new ();
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
   adw_combo_row_set_model (self->power_combo, power_profiles);
@@ -536,10 +624,36 @@ sysprof_greeter_init (SysprofGreeter *self)
                            create_envvar_row_cb,
                            self, NULL);
 
+  if (self->recording_template)
+    {
+      g_auto(GStrv) environ = NULL;
+      g_object_get (self->recording_template,
+                    "environ", &environ,
+                    NULL);
+      if (environ)
+        {
+          for (guint i = 0; environ[i]; i++)
+            gtk_string_list_append (self->envvars, environ[i]);            
+        }
+    }
+
   row = gtk_list_box_get_row_at_index (self->sidebar_list_box, 0);
   gtk_list_box_select_row (self->sidebar_list_box, row);
   sidebar_row_activated_cb (self, row, self->sidebar_list_box);
 
+  g_signal_connect_object (self->sample_user_stack_size,
+                           "notify::selected-item",
+                           G_CALLBACK (on_stack_size_changed_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  /* Set to 16KB */
+  adw_combo_row_set_selected (self->sample_user_stack_size, 1);
+
+#if !defined(__x86_64__) && !defined(__i386__)
+  gtk_widget_set_visible (GTK_WIDGET (self->user_stacks), FALSE);
+  gtk_widget_set_visible (GTK_WIDGET (self->user_stacks_caption), FALSE);
+#endif
+
   gtk_widget_grab_focus (GTK_WIDGET (self->record_to_memory));
 }
 
