@@ -20,7 +20,12 @@
 
 #include "config.h"
 
+#include <json-glib/json-glib.h>
+
 #include "sysprof-recording-template.h"
+#include "sysprof-util.h"
+
+#define DEFAULT_STACK_SIZE (4096*4)
 
 struct _SysprofRecordingTemplate
 {
@@ -31,8 +36,11 @@ struct _SysprofRecordingTemplate
   char *power_profile;
   char **environ;
 
+  guint stack_size;
+
   guint battery_charge : 1;
   guint bundle_symbols : 1;
+  guint debuginfod : 1;
   guint clear_environ : 1;
   guint cpu_usage : 1;
   guint disk_usage : 1;
@@ -49,12 +57,14 @@ struct _SysprofRecordingTemplate
   guint session_bus : 1;
   guint system_bus : 1;
   guint system_log : 1;
+  guint user_stacks : 1;
 };
 
 enum {
   PROP_0,
   PROP_BATTERY_CHARGE,
   PROP_BUNDLE_SYMBOLS,
+  PROP_DEBUGINFOD,
   PROP_CLEAR_ENVIRON,
   PROP_COMMAND_LINE,
   PROP_CPU_USAGE,
@@ -73,8 +83,10 @@ enum {
   PROP_POWER_PROFILE,
   PROP_SCHEDULER_DETAILS,
   PROP_SESSION_BUS,
+  PROP_STACK_SIZE,
   PROP_SYSTEM_BUS,
   PROP_SYSTEM_LOG,
+  PROP_USER_STACKS,
   N_PROPS
 };
 
@@ -112,6 +124,10 @@ sysprof_recording_template_get_property (GObject    *object,
     case PROP_BUNDLE_SYMBOLS:
       g_value_set_boolean (value, self->bundle_symbols);
       break;
+    
+    case PROP_DEBUGINFOD:
+      g_value_set_boolean (value, self->debuginfod);
+      break;
 
     case PROP_CLEAR_ENVIRON:
       g_value_set_boolean (value, self->clear_environ);
@@ -185,6 +201,10 @@ sysprof_recording_template_get_property (GObject    *object,
       g_value_set_boolean (value, self->session_bus);
       break;
 
+    case PROP_STACK_SIZE:
+      g_value_set_uint (value, self->stack_size);
+      break;
+
     case PROP_SYSTEM_BUS:
       g_value_set_boolean (value, self->system_bus);
       break;
@@ -193,6 +213,10 @@ sysprof_recording_template_get_property (GObject    *object,
       g_value_set_boolean (value, self->system_log);
       break;
 
+    case PROP_USER_STACKS:
+      g_value_set_boolean (value, self->user_stacks);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -215,6 +239,10 @@ sysprof_recording_template_set_property (GObject      *object,
     case PROP_BUNDLE_SYMBOLS:
       self->bundle_symbols = g_value_get_boolean (value);
       break;
+    
+    case PROP_DEBUGINFOD:
+      self->debuginfod = g_value_get_boolean (value);
+      break;
 
     case PROP_CLEAR_ENVIRON:
       self->clear_environ = g_value_get_boolean (value);
@@ -289,6 +317,10 @@ sysprof_recording_template_set_property (GObject      *object,
       self->session_bus = g_value_get_boolean (value);
       break;
 
+    case PROP_STACK_SIZE:
+      self->stack_size = g_value_get_uint (value);
+      break;
+
     case PROP_SYSTEM_BUS:
       self->system_bus = g_value_get_boolean (value);
       break;
@@ -297,6 +329,10 @@ sysprof_recording_template_set_property (GObject      *object,
       self->system_log = g_value_get_boolean (value);
       break;
 
+    case PROP_USER_STACKS:
+      self->user_stacks = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -320,6 +356,11 @@ sysprof_recording_template_class_init (SysprofRecordingTemplateClass *klass)
     g_param_spec_boolean ("bundle-symbols", NULL, NULL,
                           TRUE,
                           (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+                      
+  properties[PROP_DEBUGINFOD] =
+    g_param_spec_boolean ("debuginfod", NULL, NULL,
+                          TRUE,
+                          (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   properties[PROP_CLEAR_ENVIRON] =
     g_param_spec_boolean ("clear-environ", NULL, NULL,
@@ -421,6 +462,16 @@ sysprof_recording_template_class_init (SysprofRecordingTemplateClass *klass)
                           TRUE,
                           (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  properties[PROP_USER_STACKS] =
+    g_param_spec_boolean ("user-stacks", NULL, NULL,
+                          FALSE,
+                          (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_STACK_SIZE] =
+    g_param_spec_uint ("stack-size", NULL, NULL,
+                       0, G_MAXUINT, DEFAULT_STACK_SIZE,
+                       (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
@@ -428,6 +479,7 @@ static void
 sysprof_recording_template_init (SysprofRecordingTemplate *self)
 {
   self->bundle_symbols = TRUE;
+  self->debuginfod = TRUE;
   self->cpu_usage = TRUE;
   self->disk_usage = TRUE;
   self->graphics_info = TRUE;
@@ -439,6 +491,8 @@ sysprof_recording_template_init (SysprofRecordingTemplate *self)
   self->system_log = TRUE;
   self->command_line = g_strdup ("");
   self->cwd = g_strdup("");
+  self->environ = g_strdupv((char * []) { NULL });
+  self->stack_size = DEFAULT_STACK_SIZE;
 }
 
 SysprofRecordingTemplate *
@@ -571,7 +625,7 @@ sysprof_recording_template_apply (SysprofRecordingTemplate  *self,
 
   if (self->bundle_symbols)
     sysprof_profiler_add_instrument (profiler, sysprof_symbols_bundle_new ());
-
+  
   if (self->cpu_usage)
     sysprof_profiler_add_instrument (profiler, sysprof_cpu_usage_new ());
 
@@ -619,7 +673,12 @@ sysprof_recording_template_apply (SysprofRecordingTemplate  *self,
     sysprof_profiler_add_instrument (profiler, sysprof_memory_usage_new ());
 
   if (self->native_stacks)
-    sysprof_profiler_add_instrument (profiler, sysprof_sampler_new ());
+    {
+      if (self->user_stacks)
+        sysprof_profiler_add_instrument (profiler, sysprof_user_sampler_new (self->stack_size));
+      else
+        sysprof_profiler_add_instrument (profiler, sysprof_sampler_new ());
+    }
 
   if (self->network_usage)
     sysprof_profiler_add_instrument (profiler, sysprof_network_usage_new ());
@@ -640,3 +699,114 @@ sysprof_recording_template_apply (SysprofRecordingTemplate  *self,
 }
 
 G_DEFINE_QUARK (SysprofRecordingTemplateError, sysprof_recording_template_error)
+
+SysprofRecordingTemplate *
+sysprof_recording_template_new_from_file (GFile   *file,
+                                          GError **error)
+{
+  g_autoptr(JsonParser) parser = NULL;
+  g_autoptr(GFile) state_file = NULL;
+  SysprofRecordingTemplate *self;
+  JsonNode *root;
+
+  g_return_val_if_fail (!file || G_IS_FILE (file), NULL);
+
+  if (file == NULL)
+    file = state_file = _get_default_state_file ();
+
+  parser = json_parser_new ();
+
+  if (g_file_is_native (file))
+    {
+      g_autofree char *path = g_file_get_path (file);
+
+      if (!json_parser_load_from_file (parser, path, error))
+        return NULL;
+    }
+  else
+    {
+      g_autoptr(GFileInputStream) stream = g_file_read (file, NULL, error);
+
+      if (stream == NULL)
+        return NULL;
+
+      if (!json_parser_load_from_stream (parser, G_INPUT_STREAM (stream), NULL, error))
+        return NULL;
+    }
+
+  root = json_parser_get_root (parser);
+
+  if (!(self = SYSPROF_RECORDING_TEMPLATE (json_gobject_deserialize (SYSPROF_TYPE_RECORDING_TEMPLATE, root))))
+    return NULL;
+
+  return g_steal_pointer (&self);
+}
+
+gboolean
+sysprof_recording_template_save (SysprofRecordingTemplate  *self,
+                                 GFile                     *file,
+                                 GError                   **error)
+{
+  g_autoptr(GFileOutputStream) stream = NULL;
+  g_autoptr(JsonGenerator) generator = NULL;
+  g_autoptr(JsonNode) root = NULL;
+
+  g_return_val_if_fail (SYSPROF_IS_RECORDING_TEMPLATE (self), FALSE);
+  g_return_val_if_fail (G_IS_FILE (file), FALSE);
+
+  if (!(stream = g_file_replace (file, NULL, FALSE, G_FILE_CREATE_REPLACE_DESTINATION, NULL, error)))
+    return FALSE;
+
+  root = json_gobject_serialize (G_OBJECT (self));
+  generator = json_generator_new ();
+
+  json_generator_set_root (generator, root);
+  json_generator_set_pretty (generator, TRUE);
+
+  return json_generator_to_stream (generator, G_OUTPUT_STREAM (stream), NULL, error);
+}
+
+SysprofDocumentLoader *
+sysprof_recording_template_create_loader (SysprofRecordingTemplate  *self,
+                                          int                        fd,
+                                          GError                   **error)
+{
+  g_autoptr(SysprofDocumentLoader) loader = NULL;
+  g_autoptr(SysprofMultiSymbolizer) multi = NULL;
+  g_autoptr(SysprofElfSymbolizer) elf = NULL;
+
+  g_return_val_if_fail (SYSPROF_IS_RECORDING_TEMPLATE (self), NULL);
+
+  if (!(loader = sysprof_document_loader_new_for_fd (fd, error)))
+    return NULL;
+
+  multi = sysprof_multi_symbolizer_new ();
+
+  elf = SYSPROF_ELF_SYMBOLIZER (sysprof_elf_symbolizer_new ());
+  /* TODO: add extra-debug-directories property and use that to
+   *       call sysprof_elf_symbolizer_set_external_debug_dirs(elf,...)
+   */
+
+  /* Add in order of priority */
+  sysprof_multi_symbolizer_take (multi, sysprof_bundled_symbolizer_new ());
+  sysprof_multi_symbolizer_take (multi, sysprof_kallsyms_symbolizer_new ());
+  sysprof_multi_symbolizer_take (multi, SYSPROF_SYMBOLIZER (g_steal_pointer (&elf)));
+  sysprof_multi_symbolizer_take (multi, sysprof_jitmap_symbolizer_new ());
+
+  sysprof_document_loader_set_symbolizer (loader, SYSPROF_SYMBOLIZER (multi));
+
+#if HAVE_DEBUGINFOD
+  if (self->debuginfod)
+      {
+        g_autoptr(SysprofSymbolizer) debuginfod = NULL;
+        g_autoptr(GError) debuginfod_error = NULL;
+        
+        if (!(debuginfod = sysprof_debuginfod_symbolizer_new (&debuginfod_error)))
+          g_warning ("Failed to create debuginfod symbolizer: %s", debuginfod_error->message);
+        else
+          sysprof_multi_symbolizer_take (multi, g_steal_pointer (&debuginfod));
+      }
+#endif
+
+  return g_steal_pointer (&loader);
+}
