@@ -20,7 +20,11 @@
 
 #include "config.h"
 
+#include <fcntl.h>
+#include <errno.h>
+
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 
 #include "sysprof-counters-section.h"
 #include "sysprof-cpu-section.h"
@@ -34,33 +38,37 @@
 #include "sysprof-memory-section.h"
 #include "sysprof-metadata-section.h"
 #include "sysprof-network-section.h"
+#include "sysprof-pair.h"
 #include "sysprof-processes-section.h"
 #include "sysprof-samples-section.h"
 #include "sysprof-sidebar.h"
 #include "sysprof-storage-section.h"
+#include "sysprof-task-row.h"
 #include "sysprof-util.h"
 #include "sysprof-window.h"
 
 struct _SysprofWindow
 {
-  AdwApplicationWindow  parent_instance;
+  AdwApplicationWindow   parent_instance;
 
-  SysprofDocument      *document;
-  SysprofSession       *session;
+  SysprofDocument       *document;
+  SysprofDocumentLoader *loader;
+  SysprofSession        *session;
 
-  GtkToggleButton      *show_right_sidebar;
-  GtkWidget            *left_split_overlay;
-  GtkWidget            *right_split_overlay;
-  GtkProgressBar       *progress_bar;
-  AdwWindowTitle       *stack_title;
+  GtkToggleButton       *show_right_sidebar;
+  GtkWidget             *left_split_overlay;
+  GtkWidget             *right_split_overlay;
+  GtkProgressBar        *progress_bar;
+  AdwWindowTitle        *stack_title;
 
-  guint                 disposed : 1;
+  guint                  disposed : 1;
 };
 
 enum {
   PROP_0,
   PROP_DOCUMENT,
   PROP_IS_LOADED,
+  PROP_LOADER,
   PROP_SESSION,
   N_PROPS
 };
@@ -130,7 +138,9 @@ sysprof_window_open_file_cb (GObject      *object,
                              gpointer      user_data)
 {
   GtkFileDialog *dialog = (GtkFileDialog *)object;
-  g_autoptr(GtkWindow) transient_for = user_data;
+  g_autoptr(SysprofRecordingTemplate) template = NULL;
+  g_autoptr(GtkWindow) transient_for = NULL;
+  g_autoptr(SysprofPair) pair = user_data;
   g_autoptr(GError) error = NULL;
   g_autoptr(GFile) file = NULL;
 
@@ -138,11 +148,19 @@ sysprof_window_open_file_cb (GObject      *object,
   g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (!transient_for || GTK_IS_WINDOW (transient_for));
 
+  g_object_get (pair,
+                "first", &transient_for,
+                "second", &template,
+                NULL);
+
+  g_assert (!transient_for || GTK_IS_WIDGET (transient_for));
+  g_assert (!template || SYSPROF_IS_RECORDING_TEMPLATE (template));
+
   if ((file = gtk_file_dialog_open_finish (dialog, result, &error)))
     {
       if (g_file_is_native (file))
         {
-          sysprof_window_open (SYSPROF_APPLICATION_DEFAULT, file);
+          sysprof_window_open (SYSPROF_APPLICATION_DEFAULT, template, file);
 
           if (transient_for && !SYSPROF_IS_WINDOW (transient_for))
             gtk_window_destroy (transient_for);
@@ -165,7 +183,8 @@ sysprof_window_open_file_cb (GObject      *object,
 }
 
 void
-sysprof_window_open_file (GtkWindow *parent)
+sysprof_window_open_file (GtkWindow                *parent,
+                          SysprofRecordingTemplate *template)
 {
   g_autoptr(GtkFileDialog) dialog = NULL;
   g_autoptr(GtkFileFilter) filter = NULL;
@@ -189,7 +208,10 @@ sysprof_window_open_file (GtkWindow *parent)
                         parent,
                         NULL,
                         sysprof_window_open_file_cb,
-                        parent ? g_object_ref (parent) : NULL);
+                        g_object_new (SYSPROF_TYPE_PAIR,
+                                      "first", parent,
+                                      "second", template,
+                                      NULL));
 }
 
 static void
@@ -197,7 +219,12 @@ sysprof_window_open_capture_action (GtkWidget  *widget,
                                     const char *action_name,
                                     GVariant   *param)
 {
-  sysprof_window_open_file (GTK_WINDOW (widget));
+  g_autoptr(SysprofRecordingTemplate) template = NULL;
+
+  /* Open for default state file so we can apply settings */
+  template = sysprof_recording_template_new_from_file (NULL, NULL);
+
+  sysprof_window_open_file (GTK_WINDOW (widget), template);
 }
 
 static void
@@ -513,6 +540,10 @@ sysprof_window_get_property (GObject    *object,
       g_value_set_object (value, sysprof_window_get_document (self));
       break;
 
+    case PROP_LOADER:
+      g_value_set_object (value, self->loader);
+      break;
+
     case PROP_IS_LOADED:
       g_value_set_boolean (value, !!sysprof_window_get_document (self));
       break;
@@ -560,6 +591,11 @@ sysprof_window_class_init (SysprofWindowClass *klass)
                          SYSPROF_TYPE_DOCUMENT,
                          (G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
+  properties[PROP_LOADER] =
+    g_param_spec_object ("loader", NULL, NULL,
+                         SYSPROF_TYPE_DOCUMENT_LOADER,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   properties [PROP_IS_LOADED] =
     g_param_spec_boolean ("is-loaded", NULL, NULL,
                           FALSE,
@@ -617,6 +653,7 @@ sysprof_window_class_init (SysprofWindowClass *klass)
   g_type_ensure (SYSPROF_TYPE_SESSION);
   g_type_ensure (SYSPROF_TYPE_SYMBOL);
   g_type_ensure (SYSPROF_TYPE_SIDEBAR);
+  g_type_ensure (SYSPROF_TYPE_TASK_ROW);
 }
 
 static void
@@ -711,6 +748,9 @@ sysprof_window_load_cb (GObject      *object,
   g_binding_unbind (g_object_get_data (G_OBJECT (loader), "message-binding"));
   g_object_set_data (G_OBJECT (loader), "message-binding", NULL);
 
+  if (g_set_object (&self->loader, NULL))
+    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_LOADER]);
+
   if (!(document = sysprof_document_loader_load_finish (loader, result, &error)))
     {
       GtkWidget *dialog;
@@ -750,6 +790,8 @@ sysprof_window_create (SysprofApplication    *app,
   self = g_object_new (SYSPROF_TYPE_WINDOW,
                        "application", app,
                        NULL);
+  self->loader = g_object_ref (loader);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_LOADER]);
   g_object_bind_property (loader, "fraction",
                           self->progress_bar, "fraction",
                           G_BINDING_SYNC_CREATE);
@@ -762,25 +804,47 @@ sysprof_window_create (SysprofApplication    *app,
 }
 
 void
-sysprof_window_open (SysprofApplication *app,
-                     GFile              *file)
+sysprof_window_open (SysprofApplication       *app,
+                     SysprofRecordingTemplate *template,
+                     GFile                    *file)
 {
   g_autoptr(SysprofDocumentLoader) loader = NULL;
+  g_autoptr(SysprofRecordingTemplate) alt_template = NULL;
+  g_autoptr(GError) error = NULL;
+  g_autofd int fd = -1;
   SysprofWindow *self;
+  const char *path;
 
   g_return_if_fail (SYSPROF_IS_APPLICATION (app));
+  g_return_if_fail (!template || SYSPROF_IS_RECORDING_TEMPLATE (template));
   g_return_if_fail (G_IS_FILE (file));
 
-  if (!g_file_is_native (file) ||
-      !(loader = sysprof_document_loader_new (g_file_peek_path (file))))
+  if (template == NULL)
+    template = alt_template = sysprof_recording_template_new ();
+
+  if (!g_file_is_native (file))
     {
       g_autofree char *uri = g_file_get_uri (file);
       g_warning ("Cannot open non-native file \"%s\"", uri);
       return;
     }
 
+  path = g_file_peek_path (file);
+
+  if (-1 == (fd = open (path, O_RDONLY)))
+    {
+      int errsv = errno;
+      g_critical ("Failed to open %s: %s", path, g_strerror (errsv));
+      return;
+    }
+
+  if (!(loader = sysprof_recording_template_create_loader (template, fd, &error)))
+    {
+      g_critical ("Failed to create loader: %s", error->message);
+      return;
+    }
+
   g_application_hold (G_APPLICATION (app));
-  sysprof_window_apply_loader_settings (loader);
   self = sysprof_window_create (app, loader);
   sysprof_document_loader_load_async (loader,
                                       NULL,
@@ -790,18 +854,25 @@ sysprof_window_open (SysprofApplication *app,
 }
 
 void
-sysprof_window_open_fd (SysprofApplication *app,
-                        int                 fd)
+sysprof_window_open_fd (SysprofApplication       *app,
+                        SysprofRecordingTemplate *template,
+                        int                       fd)
 {
+  g_autoptr(SysprofRecordingTemplate) alt_template = NULL;
   g_autoptr(SysprofDocumentLoader) loader = NULL;
   g_autoptr(GError) error = NULL;
   SysprofWindow *self;
 
   g_return_if_fail (SYSPROF_IS_APPLICATION (app));
+  g_return_if_fail (!template || SYSPROF_IS_RECORDING_TEMPLATE (template));
+  g_return_if_fail (fd > -1);
 
-  if (!(loader = sysprof_document_loader_new_for_fd (fd, &error)))
+  if (template == NULL)
+    template = alt_template = sysprof_recording_template_new ();
+
+  if (!(loader = sysprof_recording_template_create_loader (template, fd, &error)))
     {
-      g_critical ("Failed to dup FD: %s", error->message);
+      g_critical ("Failed to create loader: %s", error->message);
       return;
     }
 
